package org.apache.bcel.classfile;

import java.util.stream.Stream;
import java.util.Iterator;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public class BootstrapMethods extends Attribute implements Iterable<BootstrapMethod>
{
    private BootstrapMethod[] bootstrapMethods;
    
    public BootstrapMethods(final BootstrapMethods c) {
        this(c.getNameIndex(), c.getLength(), c.getBootstrapMethods(), c.getConstantPool());
    }
    
    public BootstrapMethods(final int nameIndex, final int length, final BootstrapMethod[] bootstrapMethods, final ConstantPool constantPool) {
        super((byte)20, nameIndex, length, constantPool);
        this.bootstrapMethods = bootstrapMethods;
    }
    
    BootstrapMethods(final int nameIndex, final int length, final DataInput input, final ConstantPool constantPool) throws IOException {
        this(nameIndex, length, (BootstrapMethod[])null, constantPool);
        final int numBootstrapMethods = input.readUnsignedShort();
        this.bootstrapMethods = new BootstrapMethod[numBootstrapMethods];
        for (int i = 0; i < numBootstrapMethods; ++i) {
            this.bootstrapMethods[i] = new BootstrapMethod(input);
        }
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitBootstrapMethods(this);
    }
    
    @Override
    public BootstrapMethods copy(final ConstantPool constantPool) {
        final BootstrapMethods c = (BootstrapMethods)this.clone();
        c.bootstrapMethods = new BootstrapMethod[this.bootstrapMethods.length];
        for (int i = 0; i < this.bootstrapMethods.length; ++i) {
            c.bootstrapMethods[i] = this.bootstrapMethods[i].copy();
        }
        c.setConstantPool(constantPool);
        return c;
    }
    
    @Override
    public final void dump(final DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeShort(this.bootstrapMethods.length);
        for (final BootstrapMethod bootstrapMethod : this.bootstrapMethods) {
            bootstrapMethod.dump(file);
        }
    }
    
    public final BootstrapMethod[] getBootstrapMethods() {
        return this.bootstrapMethods;
    }
    
    @Override
    public Iterator<BootstrapMethod> iterator() {
        return Stream.of(this.bootstrapMethods).iterator();
    }
    
    public final void setBootstrapMethods(final BootstrapMethod[] bootstrapMethods) {
        this.bootstrapMethods = bootstrapMethods;
    }
    
    @Override
    public final String toString() {
        final StringBuilder buf = new StringBuilder();
        buf.append("BootstrapMethods(");
        buf.append(this.bootstrapMethods.length);
        buf.append("):");
        for (int i = 0; i < this.bootstrapMethods.length; ++i) {
            buf.append("\n");
            final int start = buf.length();
            buf.append("  ").append(i).append(": ");
            final int indentCount = buf.length() - start;
            final String[] lines = this.bootstrapMethods[i].toString(super.getConstantPool()).split("\\r?\\n");
            buf.append(lines[0]);
            for (int j = 1; j < lines.length; ++j) {
                buf.append("\n").append("          ", 0, indentCount).append(lines[j]);
            }
        }
        return buf.toString();
    }
}
